package com.sun.org.apache.bcel.internal.generic;

import java.util.Collection;
import java.util.HashMap;
import com.sun.org.apache.bcel.internal.classfile.Utility;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class InstructionHandle
{
    private InstructionHandle next;
    private InstructionHandle prev;
    private Instruction instruction;
    private int i_position;
    private Set<InstructionTargeter> targeters;
    private Map<Object, Object> attributes;
    
    @Deprecated
    protected void addHandle() {
    }
    
    public final InstructionHandle getNext() {
        return this.next;
    }
    
    public final InstructionHandle getPrev() {
        return this.prev;
    }
    
    public final Instruction getInstruction() {
        return this.instruction;
    }
    
    public void setInstruction(Instruction i) {
        if (i == null) {
            throw new ClassGenException("Assigning null to handle");
        }
        if (this.getClass() != BranchHandle.class && i instanceof BranchInstruction) {
            throw new ClassGenException("Assigning branch instruction " + i + " to plain handle");
        }
        if (this.instruction != null) {
            this.instruction.dispose();
        }
        this.instruction = i;
        return;
    }
    
    public Instruction swapInstruction(Instruction i) {
        Instruction oldInstruction = this.instruction;
        this.instruction = i;
        return oldInstruction;
    }
    
    protected InstructionHandle(Instruction i) {
        this.i_position = -1;
        this.setInstruction(i);
    }
    
    static InstructionHandle getInstructionHandle(Instruction i) {
        return new InstructionHandle(i);
    }
    
    protected int updatePosition(int offset, int max_offset) {
        this.i_position += offset;
        return 0;
    }
    
    public int getPosition() {
        return this.i_position;
    }
    
    void setPosition(int pos) {
        this.i_position = pos;
    }
    
    void dispose() {
        InstructionHandle instructionHandle = null;
        this.prev = instructionHandle;
        this.next = instructionHandle;
        this.instruction.dispose();
        this.instruction = null;
        this.i_position = -1;
        this.attributes = null;
        this.removeAllTargeters();
    }
    
    public void removeAllTargeters() {
        if (this.targeters != null) {
            this.targeters.clear();
        }
    }
    
    public void removeTargeter(InstructionTargeter t) {
        if (this.targeters != null) {
            this.targeters.remove(t);
        }
    }
    
    public void addTargeter(InstructionTargeter t) {
        if (this.targeters == null) {
            this.targeters = new HashSet();
        }
        this.targeters.add(t);
    }
    
    public boolean hasTargeters() {
        return this.targeters != null && this.targeters.size() > 0;
    }
    
    public InstructionTargeter[] getTargeters() {
        if (!this.hasTargeters()) {
            return new InstructionTargeter[0];
        }
        InstructionTargeter[] t = new InstructionTargeter[this.targeters.size()];
        this.targeters.toArray(t);
        return t;
    }
    
    public String toString(boolean verbose) {
        return Utility.format(this.i_position, 4, false, ' ') + ": " + this.instruction.toString(verbose);
    }
    
    @Override
    public String toString() {
        return this.toString(true);
    }
    
    public void addAttribute(Object key, Object attr) {
        if (this.attributes == null) {
            this.attributes = new HashMap(3);
        }
        this.attributes.put(key, attr);
    }
    
    public void removeAttribute(Object key) {
        if (this.attributes != null) {
            this.attributes.remove(key);
        }
    }
    
    public Object getAttribute(Object key) {
        if (this.attributes != null) {
            return this.attributes.get(key);
        }
        return null;
    }
    
    public Collection<Object> getAttributes() {
        if (this.attributes == null) {
            this.attributes = new HashMap(3);
        }
        return this.attributes.values();
    }
    
    public void accept(Visitor v) {
        this.instruction.accept(v);
    }
    
    final InstructionHandle setNext(InstructionHandle next) {
        return this.next = next;
    }
    
    final InstructionHandle setPrev(InstructionHandle prev) {
        return this.prev = prev;
    }
}
